<?php

namespace Drupal\print_this_page\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'print_this_page' formatter.
 *
 * @FieldFormatter(
 *   id = "print_this_page",
 *   label = @Translation("Print This Page"),
 *   field_types = {
 *     "boolean",
 *     "string",
 *     "text",
 *     "text_long",
 *     "text_with_summary",
 *     "entity_reference",
 *   },
 *   weight = 10
 * )
 */
class PrintThisPageFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'link_text' => 'Print this page',
      'exclude_list' => 'header, footer, aside, nav, form, iframe, .menu',
      'show_icon' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link Text'),
      '#description' => $this->t('The text that will be displayed for the print link.'),
      '#default_value' => $this->getSetting('link_text'),
      '#required' => TRUE,
    ];

    $form['show_icon'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show print icon'),
      '#description' => $this->t('Display the print icon (🖨️) next to the link text.'),
      '#default_value' => $this->getSetting('show_icon'),
    ];

    $form['exclude_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Elements to exclude from printing'),
      '#description' => $this->t('Enter a comma-separated list of CSS selectors (IDs, classes, or element names) to hide when printing. Example: header, footer, aside, nav, form, iframe, .menu'),
      '#default_value' => $this->getSetting('exclude_list'),
      '#rows' => 3,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Link text: @text', ['@text' => $this->getSetting('link_text')]);
    $summary[] = $this->t('Show icon: @show', ['@show' => $this->getSetting('show_icon') ? $this->t('Yes') : $this->t('No')]);
    $summary[] = $this->t('Elements excluded: @list', ['@list' => $this->getSetting('exclude_list')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    // We only need to render this once regardless of field values.
    if (!empty($items)) {
      $elements[0] = [
        '#theme' => 'print_this_page_button',
        '#link_text' => $this->getSetting('link_text'),
        '#show_icon' => $this->getSetting('show_icon'),
        '#attached' => [
          'library' => [
            'print_this_page/print-button',
          ],
          'drupalSettings' => [
            'printThisPage' => [
              'excludeList' => $this->getSetting('exclude_list'),
            ],
          ],
        ],
        '#cache' => [
          'contexts' => ['url'],
        ],
      ];
    }

    return $elements;
  }

}
