<?php

namespace Drupal\Tests\print_this_page\Unit\Plugin\Block;

use Drupal\print_this_page\Plugin\Block\PrintThisPageBlock;
use Drupal\Tests\UnitTestCase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\DependencyInjection\ContainerBuilder;

/**
 * Tests the PrintThisPageBlock plugin.
 *
 * @coversDefaultClass \Drupal\print_this_page\Plugin\Block\PrintThisPageBlock
 * @group print_this_page
 */
class PrintThisPageBlockTest extends UnitTestCase {

  /**
   * The plugin instance.
   *
   * @var \Drupal\print_this_page\Plugin\Block\PrintThisPageBlock
   */
  protected $plugin;

  /**
   * The mocked string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $stringTranslation;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->stringTranslation = $this->createMock(TranslationInterface::class);
    $this->stringTranslation->method('translate')
      ->willReturnCallback(function ($string, $args = [], $options = []) {
        return $string;
      });

    $container = new ContainerBuilder();
    $container->set('string_translation', $this->stringTranslation);
    \Drupal::setContainer($container);

    $configuration = [];
    $plugin_id = 'print_this_page_block';
    $plugin_definition = [
      'admin_label' => 'Print This Page',
      'category' => 'Custom',
      'provider' => 'print_this_page',
    ];

    $this->plugin = new PrintThisPageBlock(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
    $this->plugin->setStringTranslation($this->stringTranslation);
  }

  /**
   * Tests the default configuration.
   *
   * @covers ::defaultConfiguration
   */
  public function testDefaultConfiguration() {
    $expected = [
      'link_text' => 'Print this page',
      'exclude_list' => 'header, footer, aside, nav, form, iframe, .menu',
      'show_icon' => TRUE,
    ];

    $default_config = $this->plugin->defaultConfiguration();

    // For testing purposes, let's test the actual string value.
    $this->assertEquals($expected['link_text'], $default_config['link_text']->getUntranslatedString());
    $this->assertEquals($expected['exclude_list'], $default_config['exclude_list']);
    $this->assertEquals($expected['show_icon'], $default_config['show_icon']);
  }

  /**
   * Tests the block form.
   *
   * @covers ::blockForm
   */
  public function testBlockForm() {
    $form = [];
    $form_state = $this->createMock(FormStateInterface::class);

    $result = $this->plugin->blockForm($form, $form_state);

    // Test that all expected form elements are present.
    $this->assertArrayHasKey('link_text', $result);
    $this->assertArrayHasKey('show_icon', $result);
    $this->assertArrayHasKey('exclude_list', $result);

    // Test link_text field.
    $this->assertEquals('textfield', $result['link_text']['#type']);
    $this->assertEquals('Link Text', $result['link_text']['#title']->getUntranslatedString());
    $this->assertTrue($result['link_text']['#required']);
    $this->assertEquals('Print this page', $result['link_text']['#default_value']->getUntranslatedString());

    // Test show_icon field.
    $this->assertEquals('checkbox', $result['show_icon']['#type']);
    $this->assertEquals('Show print icon', $result['show_icon']['#title']->getUntranslatedString());
    $this->assertTrue($result['show_icon']['#default_value']);

    // Test exclude_list field.
    $this->assertEquals('textarea', $result['exclude_list']['#type']);
    $this->assertEquals('Elements to exclude from printing', $result['exclude_list']['#title']->getUntranslatedString());
    $this->assertEquals(3, $result['exclude_list']['#rows']);
    $this->assertEquals('header, footer, aside, nav, form, iframe, .menu', $result['exclude_list']['#default_value']);
  }

  /**
   * Tests block form submission.
   *
   * @covers ::blockSubmit
   */
  public function testBlockSubmit() {
    $form = [];
    $form_state = $this->createMock(FormStateInterface::class);

    // Be more explicit about what getValue should return.
    $form_state->expects($this->exactly(3))
      ->method('getValue')
      ->willReturnCallback(function ($key) {
        switch ($key) {
          case 'link_text':
            return 'Custom Print Text';

          case 'show_icon':
            return FALSE;

          case 'exclude_list':
            return 'header, .custom-class';

          default:
            return NULL;
        }
      });

    // Start with the plugin's default configuration.
    $this->plugin->setConfiguration($this->plugin->defaultConfiguration());

    $this->plugin->blockSubmit($form, $form_state);

    $configuration = $this->plugin->getConfiguration();
    $this->assertEquals('Custom Print Text', $configuration['link_text']);
    $this->assertFalse($configuration['show_icon']);
    $this->assertEquals('header, .custom-class', $configuration['exclude_list']);
  }

  /**
   * Tests the build method.
   *
   * @covers ::build
   */
  public function testBuild() {
    // Set custom configuration.
    $this->plugin->setConfiguration([
      'link_text' => 'Test Print Link',
      'show_icon' => FALSE,
      'exclude_list' => 'header, .test-class',
    ]);

    $build = $this->plugin->build();

    // Test theme.
    $this->assertEquals('print_this_page_button', $build['#theme']);

    // Test variables.
    $this->assertEquals('Test Print Link', $build['#link_text']);
    $this->assertFalse($build['#show_icon']);

    // Test attached library.
    $this->assertArrayHasKey('library', $build['#attached']);
    $this->assertContains('print_this_page/print-button', $build['#attached']['library']);

    // Test drupalSettings.
    $this->assertArrayHasKey('drupalSettings', $build['#attached']);
    $this->assertArrayHasKey('printThisPage', $build['#attached']['drupalSettings']);
    $this->assertEquals('header, .test-class', $build['#attached']['drupalSettings']['printThisPage']['excludeList']);

    // Test cache contexts.
    $this->assertArrayHasKey('contexts', $build['#cache']);
    $this->assertContains('url', $build['#cache']['contexts']);
  }

  /**
   * Tests build method with default configuration.
   *
   * @covers ::build
   */
  public function testBuildWithDefaults() {
    $build = $this->plugin->build();

    $this->assertEquals('print_this_page_button', $build['#theme']);
    $this->assertEquals('Print this page', $build['#link_text']->getUntranslatedString());
    $this->assertTrue($build['#show_icon']);
    $this->assertEquals('header, footer, aside, nav, form, iframe, .menu', $build['#attached']['drupalSettings']['printThisPage']['excludeList']);
  }

}
