<?php

namespace Drupal\Tests\print_this_page\Unit\Plugin\Field\FieldFormatter;

use Drupal\print_this_page\Plugin\Field\FieldFormatter\PrintThisPageFormatter;
use Drupal\Tests\UnitTestCase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\DependencyInjection\ContainerBuilder;

/**
 * Tests the PrintThisPageFormatter plugin.
 *
 * @coversDefaultClass \Drupal\print_this_page\Plugin\Field\FieldFormatter\PrintThisPageFormatter
 * @group print_this_page
 */
class PrintThisPageFormatterTest extends UnitTestCase {

  /**
   * The formatter instance.
   *
   * @var \Drupal\print_this_page\Plugin\Field\FieldFormatter\PrintThisPageFormatter
   */
  protected $formatter;

  /**
   * The mocked string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $stringTranslation;

  /**
   * The mocked field definition.
   *
   * @var \Drupal\Core\Field\FieldDefinitionInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $fieldDefinition;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->stringTranslation = $this->createMock(TranslationInterface::class);

    $this->stringTranslation->method('translate')
      ->willReturnCallback(function ($string, $args = [], $options = []) {
        return new TranslatableMarkup('@string', ['@string' => (string) $string], $options);
      });

    $this->fieldDefinition = $this->createMock(FieldDefinitionInterface::class);

    $container = new ContainerBuilder();
    $container->set('string_translation', $this->stringTranslation);
    \Drupal::setContainer($container);

    $plugin_id = 'print_this_page';
    $plugin_definition = [
      'label' => 'Print This Page',
      'field_types' => ['boolean', 'string', 'text', 'text_long', 'text_with_summary', 'entity_reference'],
    ];
    $settings = [];

    $this->formatter = new PrintThisPageFormatter(
      $plugin_id,
      $plugin_definition,
      $this->fieldDefinition,
      $settings,
      'label',
      'view_mode',
      []
    );
    $this->formatter->setStringTranslation($this->stringTranslation);
  }

  /**
   * Tests the default settings.
   *
   * @covers ::defaultSettings
   */
  public function testDefaultSettings() {
    $expected = [
      'link_text' => 'Print this page',
      'exclude_list' => 'header, footer, aside, nav, form, iframe, .menu',
      'show_icon' => TRUE,
    ];

    $default_settings = PrintThisPageFormatter::defaultSettings();

    $this->assertEquals($expected['link_text'], $default_settings['link_text']);
    $this->assertEquals($expected['exclude_list'], $default_settings['exclude_list']);
    $this->assertEquals($expected['show_icon'], $default_settings['show_icon']);
  }

  /**
   * Tests the settings form.
   *
   * @covers ::settingsForm
   */
  public function testSettingsForm() {
    $form = [];
    $form_state = $this->createMock(FormStateInterface::class);

    $result = $this->formatter->settingsForm($form, $form_state);

    // Test that all expected form elements are present.
    $this->assertArrayHasKey('link_text', $result);
    $this->assertArrayHasKey('show_icon', $result);
    $this->assertArrayHasKey('exclude_list', $result);

    // Test link_text field.
    $this->assertEquals('textfield', $result['link_text']['#type']);
    $this->assertEquals('Link Text', $result['link_text']['#title']->getUntranslatedString());
    $this->assertTrue($result['link_text']['#required']);
    $this->assertEquals('Print this page', $result['link_text']['#default_value']);

    // Test show_icon field.
    $this->assertEquals('checkbox', $result['show_icon']['#type']);
    $this->assertEquals('Show print icon', $result['show_icon']['#title']->getUntranslatedString());
    $this->assertTrue($result['show_icon']['#default_value']);

    // Test exclude_list field.
    $this->assertEquals('textarea', $result['exclude_list']['#type']);
    $this->assertEquals('Elements to exclude from printing', $result['exclude_list']['#title']->getUntranslatedString());
    $this->assertEquals(3, $result['exclude_list']['#rows']);
    $this->assertEquals('header, footer, aside, nav, form, iframe, .menu', $result['exclude_list']['#default_value']);
  }

  /**
   * Tests the settings summary.
   *
   * @covers ::settingsSummary
   */
  public function testSettingsSummary() {
    // Test with default settings.
    $summary = $this->formatter->settingsSummary();

    $this->assertCount(3, $summary);
    $this->assertEquals('Link text: @text', $summary[0]->getUntranslatedString());
    $this->assertEquals('Show icon: @show', $summary[1]->getUntranslatedString());
    $this->assertEquals('Elements excluded: @list', $summary[2]->getUntranslatedString());
  }

  /**
   * Tests the settings summary with custom settings.
   *
   * @covers ::settingsSummary
   */
  public function testSettingsSummaryWithCustomSettings() {
    // Create formatter with custom settings.
    $formatter = new PrintThisPageFormatter(
      'print_this_page',
      [],
      $this->fieldDefinition,
      [
        'link_text' => 'Custom Print',
        'show_icon' => FALSE,
        'exclude_list' => 'header, .custom',
      ],
      'label',
      'view_mode',
      []
    );
    $formatter->setStringTranslation($this->stringTranslation);

    $summary = $formatter->settingsSummary();

    $this->assertCount(3, $summary);

    // Check the untranslated string and arguments separately.
    $this->assertEquals('Link text: @text', $summary[0]->getUntranslatedString());
    $this->assertEquals(['@text' => 'Custom Print'], $summary[0]->getArguments());

    $this->assertEquals('Show icon: @show', $summary[1]->getUntranslatedString());
    $arguments = $summary[1]->getArguments();
    $this->assertInstanceOf(TranslatableMarkup::class, $arguments['@show']);
    $this->assertEquals('No', $arguments['@show']->getUntranslatedString());

    $this->assertEquals('Elements excluded: @list', $summary[2]->getUntranslatedString());
    $this->assertEquals(['@list' => 'header, .custom'], $summary[2]->getArguments());
  }

  /**
   * Tests view elements with field items.
   *
   * @covers ::viewElements
   */
  public function testViewElements() {
    $items = $this->createMock(FieldItemListInterface::class);
    $items->method('count')->willReturn(1);

    $elements = $this->formatter->viewElements($items, 'en');

    $this->assertArrayHasKey(0, $elements);

    $element = $elements[0];

    // Test theme.
    $this->assertEquals('print_this_page_button', $element['#theme']);

    // Test variables.
    $this->assertEquals('Print this page', $element['#link_text']);
    $this->assertTrue($element['#show_icon']);

    // Test attached library.
    $this->assertArrayHasKey('library', $element['#attached']);
    $this->assertContains('print_this_page/print-button', $element['#attached']['library']);

    // Test drupalSettings.
    $this->assertArrayHasKey('drupalSettings', $element['#attached']);
    $this->assertArrayHasKey('printThisPage', $element['#attached']['drupalSettings']);
    $this->assertEquals('header, footer, aside, nav, form, iframe, .menu', $element['#attached']['drupalSettings']['printThisPage']['excludeList']);

    // Test cache contexts.
    $this->assertArrayHasKey('contexts', $element['#cache']);
    $this->assertContains('url', $element['#cache']['contexts']);
  }

  /**
   * Tests view elements with custom settings.
   *
   * @covers ::viewElements
   */
  public function testViewElementsWithCustomSettings() {
    $formatter = new PrintThisPageFormatter(
      'print_this_page',
      [],
      $this->fieldDefinition,
      [
        'link_text' => 'Custom Print Text',
        'show_icon' => FALSE,
        'exclude_list' => 'header, .test',
      ],
      'label',
      'view_mode',
      []
    );
    $formatter->setStringTranslation($this->stringTranslation);

    $items = $this->createMock(FieldItemListInterface::class);
    $items->method('count')->willReturn(1);

    $elements = $formatter->viewElements($items, 'en');

    $this->assertArrayHasKey(0, $elements);

    $element = $elements[0];

    // Test custom variables.
    $this->assertEquals('Custom Print Text', $element['#link_text']);
    $this->assertFalse($element['#show_icon']);
    $this->assertEquals('header, .test', $element['#attached']['drupalSettings']['printThisPage']['excludeList']);
  }

}
