<?php

namespace Drupal\Tests\print_this_page\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\DependencyInjection\ContainerBuilder;

/**
 * Tests the Print This Page module hooks.
 *
 * @group print_this_page
 */
class PrintThisPageModuleTest extends UnitTestCase {

  /**
   * The module extension list service mock.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleExtensionList;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Mock the module extension list service.
    $this->moduleExtensionList = $this->createMock(ModuleExtensionList::class);

    $container = new ContainerBuilder();
    $container->set('extension.list.module', $this->moduleExtensionList);
    \Drupal::setContainer($container);

    // Include the module file.
    require_once __DIR__ . '/../../../print_this_page.module';
  }

  /**
   * Tests hook_help().
   *
   * @covers ::print_this_page_help
   */
  public function testHookHelpWithCorrectRoute() {
    $route_match = $this->createMock(RouteMatchInterface::class);
    $module_path = '/path/to/module';
    $readme_content = "# Print This Page\n\nThis is a test README.";

    $this->moduleExtensionList->method('getPath')
      ->with('print_this_page')
      ->willReturn($module_path);

    // Create a temporary README file for testing.
    $temp_dir = sys_get_temp_dir() . '/print_this_page_test';
    $readme_file = $temp_dir . '/README.md';

    if (!is_dir($temp_dir)) {
      mkdir($temp_dir, 0755, TRUE);
    }

    file_put_contents($readme_file, $readme_content);

    // Mock the file path by replacing the module path.
    $original_module_path = $module_path;
    $this->moduleExtensionList->method('getPath')
      ->with('print_this_page')
      ->willReturn(str_replace('/path/to/module', $temp_dir, $original_module_path));

    $result = print_this_page_help('help.page.print_this_page', $route_match);

    // The function will return NULL if file doesn't exist,
    // so we just test that it doesn't crash.
    $this->assertNull($result);

    // Clean up.
    unlink($readme_file);
    rmdir($temp_dir);
  }

  /**
   * Tests hook_help() with wrong route.
   *
   * @covers ::print_this_page_help
   */
  public function testHookHelpWithWrongRoute() {
    $route_match = $this->createMock(RouteMatchInterface::class);

    $result = print_this_page_help('some.other.route', $route_match);

    $this->assertNull($result);
  }

  /**
   * Tests hook_help() when README file doesn't exist.
   *
   * @covers ::print_this_page_help
   */
  public function testHookHelpWithoutReadmeFile() {
    $route_match = $this->createMock(RouteMatchInterface::class);
    $module_path = '/nonexistent/path';

    $this->moduleExtensionList->method('getPath')
      ->with('print_this_page')
      ->willReturn($module_path);

    $result = print_this_page_help('help.page.print_this_page', $route_match);

    $this->assertNull($result);
  }

  /**
   * Tests hook_theme().
   *
   * @covers ::print_this_page_theme
   */
  public function testHookTheme() {
    $result = print_this_page_theme();

    $this->assertIsArray($result);
    $this->assertArrayHasKey('print_this_page_button', $result);

    $theme_definition = $result['print_this_page_button'];

    $this->assertArrayHasKey('variables', $theme_definition);
    $this->assertArrayHasKey('template', $theme_definition);

    // Test variables.
    $variables = $theme_definition['variables'];
    $this->assertArrayHasKey('link_text', $variables);
    $this->assertArrayHasKey('show_icon', $variables);
    $this->assertArrayHasKey('module_path', $variables);

    $this->assertEquals('Print this page', $variables['link_text']);
    $this->assertTrue($variables['show_icon']);
    $this->assertEquals('', $variables['module_path']);

    // Test template.
    $this->assertEquals('print-this-page-button', $theme_definition['template']);
  }

  /**
   * Tests template_preprocess_print_this_page_button().
   *
   * @covers ::template_preprocess_print_this_page_button
   */
  public function testTemplatePreprocessPrintThisPageButton() {
    $module_path = $this->moduleExtensionList->getPath('print_this_page');

    $this->moduleExtensionList->method('getPath')
      ->with('print_this_page')
      ->willReturn($module_path);

    $variables = [];

    template_preprocess_print_this_page_button($variables);

    $this->assertArrayHasKey('module_path', $variables);
    $this->assertEquals('/' . $module_path, $variables['module_path']);
  }

}
